//+------------------------------------------------------------------+
//|                                       CloseByProfitAllOrders.mq4 |
//+------------------------------------------------------------------+
#property copyright "Copyright  2012, S.Boykov (Mislaid)"
#property link      "E-mail: fort-ino@yandex.ru"

extern double profit = 2000;
extern double stoploss = -2000;

string SymbolList = "";   //  ,    
string ProfitCheck;
int dimSymbolList = -1;   //   ,   0
double SumPosition[28][2]; //   ; - 
int WaitSeconds = 5;
int Slippage = 5;
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//----
   InitiateGlobalVariable( "LastTradeTime", 0.1);
   ProfitCheck = "Profit" + AccountNumber();
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
//----
   if ( CheckProfit() ) return(0);
   GetSymbolList(); //       
   if ( dimSymbolList >= 0) //    
      //        
      if ( AllNotZeroClosed() )
         CloseAllCrossLinked(); //    
//----
   return(0);
  }
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//|  ,     ,         |
//|    ,                                       |
//|    false,  true                         |
//+------------------------------------------------------------------+
bool CheckProfit()
{
//   ,   ()        
   if ( !FlagCheck( ProfitCheck ) )
      return( false );      
   
//     
   if ( (AccountProfit( ) >= profit) || (AccountProfit( ) < stoploss) )
//  
      if ( SetFlag( ProfitCheck ) )
         return( false);     

   return( true ); //        
}
//+---- CheckProfit() END -------------------------------------------+

//+------------------------------------------------------------------+
//|      flag                               |
//|    ,   true,  false        |
//+------------------------------------------------------------------+
bool SetFlag( string flag )
{
   int FileHandle;
   
   FileHandle = FileOpen( flag, FILE_CSV | FILE_WRITE, ";");
   if (FileHandle >= 0)
   {
      FileClose( FileHandle);
      return( true );      
   }
   else return( false );   
}
//+---- SetFlag() END -----------------------------------------------+

//+------------------------------------------------------------------+
//|       flag              |
//|    ,  true,  false              |
//+------------------------------------------------------------------+
bool FlagCheck( string flag )
{
   int FileHandle;
   
   FileHandle = FileOpen( flag, FILE_CSV | FILE_READ, ";");
   if (FileHandle >= 0)
   {
      FileClose( FileHandle);
      return( false );      
   }
   else return( true );   
}
//+---- FlagCheck() END ---------------------------------------------+

//+------------------------------------------------------------------+
//|    ,          |
//|  : double SumPosition[28,2]   |
//|     ,   -  |
//| string SymbolList   ,        |
//| int dimSymbolList   ,    |
//|    0                                         |
//+------------------------------------------------------------------+
void GetSymbolList()
{
   string SymbolName; //  
   double SumPos;    //    
   int i, NumPos; //     

//  
   ArrayInitialize( SumPosition, 0.0);
   SymbolList = "";
   dimSymbolList = -1;
      
   for ( i = OrdersTotal()-1; i>=0; i--)
      if ( OrderSelect( i, SELECT_BY_POS) )
      {
         if ( OrderType() == OP_BUY || OrderType() == OP_SELL )
         {
            SymbolName = OrderSymbol();
            NumPos = StringFind( SymbolList, SymbolName, 0);
            if ( NumPos < 0 )
            {
               //  
               dimSymbolList++;  //   
               SymbolList = SymbolList + SymbolName + " ";
               NumPos = dimSymbolList * 7;
            }
            NumPos /= 7; //    
            SumPos = SumPosition[ NumPos, 0];
            if ( OrderType() == OP_BUY  ) SumPos += OrderLots();
            if ( OrderType() == OP_SELL ) SumPos -= OrderLots();
            SumPosition[ NumPos, 0] = SumPos; //   
            SumPosition[ NumPos, 1] += 1; //    
         }
      }
   Print( SymbolList);
   
   if (dimSymbolList < 0)  //    
      FileDelete( ProfitCheck );//    
}
//+---- GetSymbolList() END -----------------------------------------+

//+------------------------------------------------------------------+
//|                          |
//|  true,        |
//|   ,  false                          |
//+------------------------------------------------------------------+
bool AllNotZeroClosed()
{
   int NumPos, NumOrd;
   double SumPos;
   double MinLot;
   string SymbolName;   
   bool result = true;
   
   for ( NumPos = 0; NumPos <= dimSymbolList; NumPos++)
   {
      SymbolName = StringSubstr( SymbolList, 7 * NumPos, 6);
      SumPos = SumPosition[ NumPos, 0];
      NumOrd = SumPosition[ NumPos, 1];   
      MinLot = MarketInfo( SymbolName, MODE_MINLOT);  
      //    
      if (MathAbs( SumPos ) > 0.5* MinLot )
      {
         result = false;
         if ( NumOrd == 1) CloseExistingOrder( SymbolName);
         else OpenNewOrder( SymbolName,  -SumPos);
      }
   }
   return( result);
}
//+---- AllNotZeroClosed() END --------------------------------------+

//+------------------------------------------------------------------+
//|                              |
//| ,                                 |
//+------------------------------------------------------------------+
void CloseExistingOrder( string SymbolName)
{
   double OrderPrice;
   
   for( int i = OrdersTotal() - 1; i >= 0; i--)         
   {
      OrderSelect( i, SELECT_BY_POS);
      if ( OrderSymbol() == SymbolName ) 
      {
         if (WaitBeforeTransaction(WaitSeconds) == 1)
         {
            RefreshRates();
            if ( OrderType() == OP_SELL ) OrderPrice = MarketInfo( SymbolName, MODE_ASK);
            else OrderPrice = MarketInfo( SymbolName, MODE_BID);
            OrderPrice = NormalizeDouble( OrderPrice, MarketInfo( SymbolName, MODE_DIGITS ));
            if ( OrderClose( OrderTicket(), OrderLots(), OrderPrice, Slippage, Yellow) )
            {
               Print(SymbolName, " OrderClose OrderTicket=", OrderTicket(),
               " OrderLots=", OrderLots(), 
               " OrderPrice=", DoubleToStr( OrderPrice, MarketInfo( SymbolName, MODE_DIGITS)) );
               break;
            }
         }
      }   
   }
}
//+---- CloseExistingOrder() END ------------------------------------+

//+------------------------------------------------------------------+
//|   ,           |
//+------------------------------------------------------------------+
void OpenNewOrder( string SymbolName, double LotSize)
{
   double OrderPrice, Lots;
   int ticket;

   Lots = NormalizeDouble( MathAbs( LotSize ), 2);

   if (WaitBeforeTransaction(WaitSeconds) == 1)
   {
      RefreshRates();
      if ( LotSize < 0 ) OrderPrice = MarketInfo( SymbolName, MODE_BID);
      else OrderPrice = MarketInfo( SymbolName, MODE_ASK);
      OrderPrice = NormalizeDouble( OrderPrice, MarketInfo( SymbolName, MODE_DIGITS ));

      if (LotSize < 0 )
      {
         ticket = OrderSend( SymbolName, OP_SELL, Lots, OrderPrice, Slippage, 0 , 0, NULL, 0, 0,  Red);
         if (ticket >= 0 )
            Print(SymbolName, " New sell order LotSize=", -LotSize, 
            " OrderPrice=", DoubleToStr( OrderPrice, MarketInfo( SymbolName, MODE_DIGITS)) );
      }
      else
      {
         ticket = OrderSend( SymbolName,  OP_BUY,  Lots, OrderPrice, Slippage, 0 , 0, NULL, 0, 0, Blue);
         if (ticket >= 0 )
            Print(SymbolName, " New buy order LotSize=", LotSize, 
            " OrderPrice=", DoubleToStr( OrderPrice, MarketInfo( SymbolName, MODE_DIGITS)) );
      }
   }
}
//+---- OpenNewOrder() END ------------------------------------------+

//+------------------------------------------------------------------+
//|                                      |
//+------------------------------------------------------------------+
void CloseAllCrossLinked()
{
   int i, NumPos, BuyTicket, SellTicket;
   double BuyLots, SellLots;
   string SymbolName;
   
   for ( NumPos = 0; NumPos <= dimSymbolList; NumPos++)
   {
      SymbolName = StringSubstr( SymbolList, 7 * NumPos, 6);   
      for( i=OrdersTotal()-1; i>=0; i--)         
      {
         OrderSelect( i, SELECT_BY_POS);
         if ( OrderSymbol() == SymbolName ) 
         {
            if ( OrderType() == OP_SELL )
            {
               SellTicket = OrderTicket();
               SellLots   = OrderLots();
            }
            if ( OrderType() == OP_BUY )
            {
               BuyTicket = OrderTicket();
               BuyLots   = OrderLots();
            }
         }
      }
      
      if (BuyLots <= SellLots)
      {
         if (OrderCloseBy( BuyTicket, SellTicket) )
            Print( SymbolName, " Lots=", BuyLots, 
                  " Close crosslinked BuyTicket=", BuyTicket, " SellTicket=", SellTicket);
      }
      else
      {
         if (OrderCloseBy( SellTicket, BuyTicket) )
            Print( SymbolName, " Lots=", SellLots,
                  " Close crosslinked SellTicket=", SellTicket, " BuyTicket=", BuyTicket);
      }
   }
}
//+---- CloseAllCrossLinked() END -----------------------------------+

//+------------------------------------------------------------------+
//|  WaitBeforeTransaction   Secs       |
//|     (  5 )      |
//| :                                                      |
//|  1 -                                  |
//|  0 -                                     |
//| -1 -   -                               |
//| Delay -  Delay*0.01 ,        |
//|                                       |
//+------------------------------------------------------------------+
int WaitBeforeTransaction(int Secs = 5, int Delay=0)
{
   //   ,   
   if (IsTesting()) return(1);
   
   int ErrorCode;
   Print("WaitBeforeTransaction started");
   
// 
   if (Delay>0) Sleep(10*Delay);
   
// ---------------------------------------------------------------
   //  Secs 
   double LastTradeTime;
   while(true)
   {
      //   ,    0
      if (IsStopped())
      {
         Print("WaitBeforeTransaction(): The expert is stopped. We leave...");
         return(0);
      }
      
      //    
      //     ,   
      //    
      LastTradeTime = GlobalVariableGet("LastTradeTime");
      //    (  ),    

      if (LastTradeTime == 0)  
      {
         ErrorCode = GetLastError();
         Print("WaitBeforeTransaction(): Error ", ErrorCode,
         " at reading a global variable LastTradeTime");
         return(-1);
      }   

      //    Secs ,  
      if ((TimeLocal() - LastTradeTime) < Secs)
      {
         //  0.1 
         Sleep(100);
         continue;
      }
      // ..   Secs ,    
      //   LastTradeTime   
      //   GlobalVariableSetOnCondition(),  
      // ,         
      //      
      if (GlobalVariableSetOnCondition("LastTradeTime", TimeLocal(), LastTradeTime))
      {
         //       , 
         //     
         //  ,   
         Print("WaitBeforeTransaction finished correctly");
         if (IsTradeContextBusy( ) ) return(-1); else return(1);
      }
      else
      {
         //      
         //   
          Print("WaitBeforeTransaction(): It was not possible to change value of a global variable. ",
                "Waiting time has expired. We leave...");
          Print(LastTradeTime);

          return(-1);
      }
   }
}
//--------------------- END WaitBeforeTransaction --------------------

//+------------------------------------------------------------------+
//|  InitiateGlobalVariable   |
//|    GlobalVariable_Name                          |
//|    ,                        |
//|     init()                        |
//+------------------------------------------------------------------+
int   InitiateGlobalVariable(string GlobalVariable_Name, double InitialValue)
{
// ----     GlobalVariable_Name ----
   int ErrorCode;

   //    GlobalVariable_Name  ,
   //   
   if (!GlobalVariableCheck("GlobalVariable_Name"))
   {
      //   -    
      // GlobalVariableCheck(),   
      ErrorCode = GetLastError();
      if (ErrorCode!=0)
      {
         Print("InitiateGlobalVariable(): Error ", ErrorCode,
         " at check of a global variable ", GlobalVariable_Name);
         return(-1);
      }
      //    ,  
      if (GlobalVariableSet("LastTradeTime", InitialValue) == 0 )
      {
         ErrorCode = GetLastError();
         if (ErrorCode!=0)      
         {
           //      
           Print("InitiateGlobalVariable(): Error ",GetLastError(),
           " at creation of a global variable ", GlobalVariable_Name);
           return(-1);
         }
      }
      else
      {
         //    
         Print("InitiateGlobalVariable(): Global variable ", GlobalVariable_Name,
         " is created");
         return(0);
      }
   }
}
//--------------------- END InitiateGlobalVariable --------------------

// the end.